package com.om.utils;


import com.om.OperationApplication;
import com.om.entity.dto.ControlListMap;
import com.om.entity.vo.RepairTitle;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Component
public class HtmlGenerator {
    @Value("${baseStatic}")
    private  String baseStatic;


    public  File generateHtmlByMap(Integer id, List<ControlListMap> controlListMap
            , Map<String, String> baseMap, Map<String, String> companyMap, RepairTitle repairTitle){

        try {
            Configuration cfg = new Configuration(Configuration.VERSION_2_3_31);
            cfg.setClassForTemplateLoading(OperationApplication.class, "../../template/static/");

            Template template = cfg.getTemplate("report.html");
            String format = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            StringWriter out = new StringWriter();
            Map<String, Object> data = new HashMap<>();

            data.put("reportTime",format);
            data.put("baseMap", baseMap);
            data.put("controlListMap", controlListMap);
            data.put("companyMap", companyMap);
            data.put("id", id);
            data.put("mainTitle",repairTitle.getMainTitle());
            data.put("carInfoTitle",repairTitle.getCarInfoTitle());
            data.put("diagnosticDetailsTitle",repairTitle.getDiagnosticDetailsTitle());
            data.put("abnormalSystemTitle",repairTitle.getAbnormalSystemTitle());
            data.put("normalSystemTitle",repairTitle.getNormalSystemTitle());
            data.put("faultCodeTitle",repairTitle.getFaultCodeTitle());
            data.put("versionInfoTitle",repairTitle.getVersionInfoTitle());

            data.put("vin",repairTitle.getVin());
            data.put("modelYear",repairTitle.getModelYear());
            data.put("mileage",repairTitle.getMileage());
            data.put("diagnosticPath",repairTitle.getDiagnosticPath());
            data.put("repairNum",repairTitle.getRepairNum());
            data.put("SN",repairTitle.getSN());
            data.put("company",repairTitle.getCompany());
            data.put("address",repairTitle.getAddress());
            data.put("telephone",repairTitle.getTelephone());
            data.put("message",repairTitle.getMessage());


            template.process(data, out);

            // 将HTML内容写入文件
            String htmlContent = out.toString();
            String yyyyMMdd = LocalDate.now().format(DateTimeFormatter.ofPattern("yyyyMMdd"));
            String outputPath = baseStatic+yyyyMMdd+"outputH5"+id+".html";
            FileWriter fileWriter = new FileWriter(outputPath);
            fileWriter.write(htmlContent);
            File file = new File(baseStatic+yyyyMMdd+"outputH5"+id+".html");
            fileWriter.close();
            System.out.println("HTML文件已生成：" + outputPath);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    public  File ziliaoHtml(String title,String pdfUrl,Integer id){

        try {
            Configuration cfg = new Configuration(Configuration.VERSION_2_3_31);
            cfg.setClassForTemplateLoading(OperationApplication.class, "../../template/static/");

            Template template = cfg.getTemplate("ziliao.html");
            StringWriter out = new StringWriter();
            Map<String, Object> data = new HashMap<>();
            data.put("title",title);
            data.put("pdfUrl", pdfUrl);
            template.process(data, out);

            // 将HTML内容写入文件
            String htmlContent = out.toString();
            String outputPath = baseStatic+"ziliao"+id+".html";
            FileWriter fileWriter = new FileWriter(outputPath);
            fileWriter.write(htmlContent);
            File file = new File(baseStatic+"ziliao"+id+".html");
            fileWriter.close();
            System.out.println("HTML文件已生成：" + outputPath);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }




}
